@@ -53,11 +53,9 @@ tlmgr = function(args = character(), usermode = FALSE, ..., .quiet = FALSE) {
 
 #' @importFrom xfun is_linux is_unix is_macos is_windows with_ext
 tweak_path = function() {
-  # check if ~/bin/tlmgr exists (created by TinyTeX by default)
-  f = if (is_linux()) '~/bin/tlmgr' else if (is_windows()) {
-    win_app_dir('TinyTeX', 'bin', 'win32', 'tlmgr.bat', error = FALSE)
-  } else if (is_macos()) '~/Library/TinyTeX/bin/x86_64-darwin/tlmgr' else return()
-  if (!file_test('-x', f)) f = getOption('tinytex.tlmgr.path', '')
+  # check tlmgr exists under the default installation dir of TinyTeX, or the
+  # global option tinytex.tlmgr.path
+  f = getOption('tinytex.tlmgr.path', find_tlmgr())
   if (!file_test('-x', f)) return()
   bin = normalizePath(dirname(f))
   # if the pdftex from TinyTeX is already on PATH, no need to adjust the PATH
@@ -116,13 +114,22 @@ tlmgr_install = function(pkgs = character(), usermode = FALSE, path = !usermode
     }
     if (Sys.which('gs') == '') warning('GhostScript is required for the epstopdf package.')
   }
-  # only run `tlmgr path add` if the symlink for tlmgr exists under
-  # /usr/local/bin; it may not exist when TinyTeX is installed with --no-path
-  if (missing(path)) path = path && file.exists('/usr/local/bin/tlmgr')
+  if (missing(path)) path = path && need_add_path()
   if (path) tlmgr_path('add')
   invisible(res)
 }
 
+# we should run `tlmgr path add` after `tlmgr install` only when the `tlmgr`
+# found from PATH is a symlink that links to another symlink (typically under
+# TinyTeX/bin/platform/tlmgr, which is typically a symlink to tlmgr.pl)
+need_add_path = function() {
+  (p <- Sys.which('tlmgr')) != '' && is_writable(p) &&
+    (p2 <- Sys.readlink(p)) != '' && basename(Sys.readlink(p2)) == 'tlmgr.pl' &&
+    basename(dirname(dirname(p2))) == 'bin'
+}
+
+is_writable = function(p) file.access(p, 2) == 0
+
 # check of certain LaTeX packages are installed: if installed, `tlmgr info pkgs`
 # should return `pkgs`
 check_installed = function(pkgs) {
